Graphics3D 800,600,32,2


Include "texture_splatting.bb"


; Player movement speed
Global PlayerSpeed#=0.1
Global inclination#=60
Global sunangle#=0.0


; Frametweening
Global GameSpeed%=60
Global Screenwidth%=GraphicsWidth()
Global Screenheight%=GraphicsHeight()
Global FramePeriod%=1000/GameSpeed
Global FrameTime%=MilliSecs()-FramePeriod


; move variables
Global Camera,SpeedMultiplicator%,DeltaTimeOld


; world
Global Mesh1%,Mesh2%,Pivot%,Light%,Sun%


; central pivot
Pivot=CreatePivot()


; Camera
Camera=CreateCamera()
PositionEntity Camera,-56,12,-28
CameraZoom Camera,1
CameraRange Camera,0.1,1024
CameraFogMode Camera,True
CameraFogRange Camera,96,192
CameraFogColor Camera,155,174,206
CameraClsColor Camera,155,174,206
PointEntity Camera,Pivot
EntityType Camera,1
EntityRadius Camera,0.15

; Light
Sun=CreateSphere(8,Pivot)
EntityFX Sun,1+8
EntityColor Sun,255,255,0
ScaleEntity Sun,20,20,20
PositionEntity Sun,512,0,0
Light=CreateLight(2,Sun)
PositionEntity Light,0,0,0
LightRange Light,1024
AmbientLight 64,64,64
LightColor Light,128,128,128


; Load maps
hm=LoadImage("maps/hm.png")	; Heightmap
cm=LoadImage("maps/cm.png")	; Colormap


; create multitextured mesh
Mesh1=CreateMesh() : h#=24 ; height multiplier


; create normals mesh to put on top
Mesh2=CreateMesh()
EntityBlend Mesh2,2
EntityType Mesh2,2

; create Meshterrain (returns surface indexes) - the order must be descending top>down with the base layer on very top
grass1lush	= LoadMeshTerrain(Mesh1,hm,h,cm,1.0,LoadImage("maps/1_grass1-lush.png"),False,False,False,True)
rock1b		= LoadMeshTerrain(Mesh1,hm,h,False,1.0,LoadImage("maps/2_rock1b.png"))
grass1		= LoadMeshTerrain(Mesh1,hm,h,False,1.0,LoadImage("maps/3_grass1.png"))
rock1		= LoadMeshTerrain(Mesh1,hm,h,False,1.0,LoadImage("maps/4_rock1.png"))
bump		= LoadMeshTerrain(Mesh2,hm,h,False,1.0,False)


; apply textures to surface indexes
ApplyTexture("textures/grass1-lush.jpg"	,0,1.0/ 32,1.0/ 32,2,False,grass1lush,2)
ApplyTexture("textures/rock1b.jpg"		,0,1.0/ 32,1.0/ 32,5,False,rock1b	,2+32)
ApplyTexture("textures/grass1.jpg"		,0,1.0/ 32,1.0/ 32,5,False,grass1	,2+32)
ApplyTexture("textures/rock1.jpg"		,0,1.0/ 32,1.0/ 32,5,False,rock1	,2+32)
ApplyTexture("maps/nm.jpg"				,0,1.0/  1,1.0/  1,4,False,bump,1)


; Apply detail texture to normals mesh
detail=LoadTexture("textures/detail.jpg") : TextureBlend detail,5 : ScaleTexture detail,1.0/64,1.0/64
EntityTexture Mesh2,detail,0,1


; centered pointer
MoveMouse GraphicsWidth()/2,GraphicsHeight()/2


Collisions 1,2,2,3

; Main loop
While Not KeyHit(1)
	
	Cls
	
	; Frametween calculation
	Local FrameElapsed%,FrameTicks%,FrameTween#,t%
	Repeat FrameElapsed=MilliSecs()-FrameTime Until FrameElapsed
	FrameTicks=FrameElapsed/FramePeriod
	FrameTween=Float(FrameElapsed Mod FramePeriod)/Float(FramePeriod)
	
	Local wf%,sl1%,sl2%
	
	; Frametween loop
	For t=1 To FrameTicks
		
		; Frametween Captureworld
		FrameTime=FrameTime+FramePeriod : If t=FrameTicks Then CaptureWorld
	
		; Space = Wireframe on/off
		If KeyHit(57) Then wf%=1-wf : WireFrame wf
		
		If KeyHit(2) Then sl1=1-sl1 : If sl1=1 Then HideEntity Mesh1 Else ShowEntity Mesh1
		If KeyHit(3) Then sl2=1-sl2 : If sl2=1 Then HideEntity Mesh2 Else ShowEntity Mesh2
		
		; Key Q/W: light angle
		If KeyDown(16) Then sunangle=sunangle+1 : If sunangle>359 Then sunangle=0
		If KeyDown(17) Then sunangle=sunangle-1 : If sunangle<0 Then sunangle=359
		
		; Key A/S: increase / decrease inclination
		If KeyDown(30) Then inclination=inclination+1 : If inclination>90 Then inclination=90
		If KeyDown(31) Then inclination=inclination-1 : If inclination<-90 Then inclination=-90
		
		; turn central pivot
		RotateEntity Pivot,0,sunangle,inclination
		
		; color normals mesh according to light position
		EntityColor Mesh2,(Cos(sunangle)+1)/2.0*255,(Sin(sunangle)+1)/2.0*255,Normalize(inclination,-90,90,0,255)
		
		; Movement
		FreeCam()
		
		UpdateWorld
		
	Next
	
	; Frametween Render
	RenderWorld FrameTween
	
	Text 0, 0,"Tris rendered..: "+TrisRendered()
	Text 0,15,"Sun Angle......: "+sunangle+""
	Text 0,30,"Sun Inclination: "+inclination+""
	
	; Bufferflip
	Flip 0
	
Wend

End


; customized Loadtexture Function
Function ApplyTexture(filename$,flags%,uscale#=1.0,vscale#=1.0,blendmode%=False,mesh%,surf%=False,fx%=False)
	
	Local tex%=LoadTexture(filename$,flags)
	ScaleTexture tex,uscale,vscale
	If blendmode Then TextureBlend tex,blendmode
	
	Local brush%=CreateBrush(255,255,255)
	If tex Then BrushTexture brush,tex
	If fx Then BrushFX brush,fx
	
	If surf Then
		PaintSurface surf,brush
	Else 
		surf=CreateSurface(mesh)
		PaintSurface surf,brush
	EndIf
	
	FreeBrush brush
	
	Return tex
	
End Function


; Freecam Movement
Function FreeCam(maxpitch#=85.0,rotspeed#=16.666,rotfloat#=8.0)
	
	Local movex#,movez#,dx#,dy#,dk#,dt%,t%
	Local movespeed#,pitch#
	
	SpeedMultiplicator=1
	
	; LMB/RMB = 2x faster
	If MouseDown(1) Or MouseDown(2) Then SpeedMultiplicator=2
	
	; Arrows = Move
	movex=KeyDown(205)-KeyDown(203)
	movez=KeyDown(200)-KeyDown(208)
	
	; Movespeed
	movespeed=PlayerSpeed*SpeedMultiplicator
	
	; smooth movement
	t=MilliSecs() : dt=t-DeltaTimeOld : DeltaTimeOld=t : dk=Float(dt)/rotspeed
	dx=(Screenwidth/2-MouseX())*0.01*dk : dy=(Screenheight/2-MouseY())*0.01*dk
	TurnEntity Camera,-dy,dx*0.1*dk*rotfloat,0
	
	; limit pitch
	pitch=EntityPitch(Camera,1) : If pitch>maxpitch Then pitch=maxpitch Else If pitch<-maxpitch Then pitch=-maxpitch
	
	; rotate and move
	RotateEntity Camera,pitch,EntityYaw(Camera,1),0,1	
	MoveEntity Camera,movex*movespeed,0,movez*movespeed
	
End Function
;~IDEal Editor Parameters:
;~C#Blitz3D